/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1  
 * 
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is C
 *
 * The Initial Developer of the Original Code is
 * Ben Reeves.
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 *
 * ***** END LICENSE BLOCK ***** */


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <compiz.h>
#include <ctype.h>

#define ICON_WIDTH 210

 GSList * active_plugins; /* List of active Plugins */
 GtkWidget *window1; /* Main window */
 GtkWidget * main_hbox; /* Everything is packed in here */
 GtkWidget *configuration_vbox; /* plugin options are packed here */
 gboolean medium_interface = TRUE; /* Where advanced options are hidden or not */
 GtkWidget *icon_selection_vbox; /* Icon browser is packed here (need to be global?) */
 GtkWidget * InterfaceVbox;
 gint number_of_cols = 3;

/* Icon selection box struct */
typedef struct {
   GtkWidget * container; // The event box is packed in here (not needed, I should remove it)
   GtkWidget * eventbox; // The icons table is packed in here (it's the white bit)
   GtkWidget * table; // Table where icons are packed - destroyed on redraw
   int current_col; // Column where last icon was packed
   int current_row; // Row where last icon was packed
   gpointer  * data; // Extra data attached to the icon group (can be anything)
   GFunc     regenerate_func; // Function that should be called when it's redrawn
   gchar     * group_name; // Name (title) of the selection
   GSList    * extra_widgets; /* Widgets that need to be destroyed when the group is recreated (not redrawn), hack for the left bar (anyone thinking of porting this code can remove it) */
   GSList * icons_list; /* List of icons/filters within the group */
   GSList * screen_options_list; /* GSlist of CompizSettingsOption's */
   GSList * allscreen_options_list;
} IconGroup;

/* This is a list of filters/icons (i.e. Keyboard Shortcuts or Fire Effect) Only the plugins icon box uses this, I havn't got round making the main select vbox use it */ 
typedef struct {
   IconGroup * icon_group; /*Pointer to icon group the filter belongs to */
   GtkWidget * widget;
   gint option_filter;
   GSList * filter_must_have;
   GSList * filter_must_not_have;
   GSList * filtered_allscreen_options_list;
   GSList * filtered_screen_options_list;
   gchar * name;
   gchar * title;
   gchar * image;
} Plugin_Filter;

/* Left bar keeps track of widget is it, allows it to be more dynamic */
typedef struct {
   GtkWidget * common_tasks;
   GtkWidget * reset_default;
} LeftVbox;
LeftVbox * left_vbox;

/* Keep track of checkbox on a Icon button */
typedef struct {
	GtkWidget * checkbox;
	gchar * plugin_name;
	gboolean checkbox_active;
} Plugin_Filter_Button;

/* Option */
typedef struct {
	gpointer * data;
	gchar * key;
	gchar * screen;
	gchar * plugin_name;
	gint type;
} CompizSettingsOption;

typedef struct {
	gchar * key;
	gchar * button;
	GSList * edge;
	int edge_button;
} CompizSettingsActionKey;

/* Plugin */
typedef struct {
	GtkWidget * widget;
	gchar * plugin_name;
	GSList * load_before;
	GSList * load_after;
	GSList * depends;
	gboolean advanced;
	gchar * description;
} CompizSettingsPlugin;

/* List of icon groups that need to be redrawn when the window is resized */
GSList * icon_groups = NULL;

GSList * Plugins_list = NULL;

GtkWidget * active_textentry = NULL;

/* Single icon group that needs to be redrawn when the window is resized */
IconGroup * plugin_sections_group;

gchar * global_icon_filter = NULL;


GdkColor color;

void activate_plugin(gchar * plugin_name);
void active_plugin_changed(gchar * plugin_name);
gint dbus_backend_get_option_type(gchar * plugin, gchar * screen, gchar * key);
void redraw_groups(GSList * icon_groups);
static int strmatch(gchar *string, gchar *stringtm);
gboolean strmatch_in_gslist(gchar *string, GSList *listtm);
void add_icon_to_group_table(gchar *data, gchar * image_path, IconGroup * new_icon_group, GFunc * click_cb, gpointer * cb_data, gboolean has_checkbox);
void icons_window_resized(GtkWidget *widget, GdkRectangle *allocation, gpointer * user_data);
void groups_window_resized(GtkWidget *widget, GdkRectangle    *allocation, gpointer *user_data);
static char * str_replace (gchar *str, const gchar *search, const gchar *replace);
GSList * get_active_plugins();
IconGroup * icon_selection_new_icon_group(gchar * title);
void plugins_loop(gchar * dir);
void setup_list_store();
GtkWidget * build_general_interface(GtkWidget *vbox);


#include "eggcellrendererkeys.h"
#include "eggaccelerators.h"
#include "eggcellrendererkeys.c"
#include "eggaccelerators.c"
#include "support.c"
#include "gconf-backend.c"
#include "dbus.c"
#include "CompizSettings-backend-functions.c"
#include "functions.c"
#include "dialogs.c"
#include "interface.h"
#include "interface.c"



int
main (int argc, char *argv[])
{

  /* Set locale */
  gtk_set_locale ();

  /* Start Gtk */
  gtk_init (&argc, &argv);

  /* Pixmaps */
  add_pixmap_directory ("/usr/share/pixmaps/compiz-settings");



  /*start dbus */
  if (!init_dbus ())	{
     dialog_dbus_error();
  }


  backend_initial_setup();
  CompizSettings_backend_setup();
  //set_interface_level ();
  active_plugins = backend_get_list_of_active_plugins();
  Plugins_list = backend_get_list_of_plugins();


#ifdef ENABLE_NLS
  bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
  textdomain (GETTEXT_PACKAGE);
#endif

  create_window1 ();
  g_signal_connect (G_OBJECT (window1), "destroy", G_CALLBACK (quit_button), NULL);
  if (!GTK_WIDGET_VISIBLE (window1))
    {
      gtk_widget_show_all (window1);
    }
  else
    {    
      gtk_widget_destroy (window1);
      window1 = NULL;
    }


  gtk_main ();
  return 0;
}

